#define FILTER_SIZE 9

__constant float GX[FILTER_SIZE] = {1,0,-1, 2,0,-2,  1, 0,-1};
__constant float GY[FILTER_SIZE] = {1,2, 1, 0,0, 0, -1,-2,-1};

kernel void edge(global float4* src, global float4* dst)
{   
    int x = get_global_id(0);
    int y = get_global_id(1);
    int w = get_global_size(0);
    int h = get_global_size(1);
    
    int N = get_global_size(0) * get_global_size(1);
    
    float3 gx = (float3)(0,0,0);
    float3 gy = (float3)(0,0,0);
    
    int i = 0;
    
    for(; i < FILTER_SIZE; ++i)
    {
        int col = (i % 3) - 1;
        int row = (i / 3) - 1;
        
        uint index = (y+col) * w + (x+row);
        
        index = index < 0 ? N-index : (index >= N ? index % N : index);
        
        float4 c = src[index];
        
        gx += GX[i] * c.xyz;
        gy += GY[i] * c.xyz;
    }
    
    float3 c = sqrt(gx*gx + gy*gy);
    float color = (c.x + c.y + c.z) / 3.0f;
    
    dst[y * w + x] = (float4)(color, color, color, 1);
} 

kernel void blur(global float4* src, global float4* dst, constant float* mask, const int maskSize, const int N, const float sum)
{   
    int x = get_global_id(0);
    int y = get_global_id(1);
    int w = get_global_size(0);
    int h = get_global_size(1);
    
    float4 color = 0;
    int halfSize = maskSize / 2;
    
    for(int i = 0; i < maskSize; ++i)
    {
        int row = i - halfSize;
        
        for(int j = 0; j < maskSize; ++j)
        {
            int col = j - halfSize;
        
            uint index = (y + row) * w + x + col;
        
            index = index < 0 ? N-index : (index >= N ? index % N : index);

            color += src[index] * mask[i * maskSize + j];
        }
    }
    
    color /= sum;
    
    dst[y * w + x] = color;
} 

